"""
    A quick script for memory profiling
    
    
"""
import gc


"""
> def calcsize(i, s=0):
>     s = sizeof(i)
>     if type(i) == type({}):
>         for k, v in i.items():
>             s += calcsize(k)
>             s += calcsize(v)
>             s += 2 * sizeof(myPyObject())

Well, the table size will usually be around 3/2 of the number
of entries. For more details see the C implementation :-)

>     elif type(i) == type([]):
>         for v in i:
>             s += calcsize(v)
>             s += 1 * sizeof(myPyObject())
>     elif type(i) == type((1,)):
>         for v in i:
>             s += calcsize(v)
>     return s
> 
"""

def dump_memory():
    """ Prints memory stats """
    objects = gc.get_objects()
    
    # __class__ -> ref count dictionary
    type_table = {}
    
    for obj in objects:
        
        if hasattr(obj, "__class__"):
            t = str(obj.__class__)
        else:
            t = str(type(obj))
                    
        if type_table.has_key(t):
            type_table[t] += 1
        else:
            type_table[t] = 1
        
        
    # make to list
    # list of tuples (__class__, count)
    
    type_tuples = []
    
    for t in type_table.keys():
        type_tuples.append((t, type_table[t]))
    
    #print str(type_tuples)
   
    # sort by reference count    
    def compare_ref_counts(x, y):
        return y[1]-x[1]
    
    type_tuples.sort(compare_ref_counts)
    
    
    print "clz:" + str(type_tuples.__class__)
    
    # Maximum number of entries to print
    max = 200
    
    for t in type_tuples:
        print str(t)
        # print "Type:" + str(type[0]) + " refs:" + str(type[1])
        max -= 1
        if max == 0:
            break
            
    
    
    