"""

  Usability application category class
  Copyright 2005 Mikko Ohtamaa

  Usability product initialization code

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as described
  in licence.txt

"""
from Products.Archetypes.public import process_types, listTypes
from Products.Archetypes.ArchetypeTool import getType
from Products.Archetypes.utils import capitalize
from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory
from AccessControl import allow_module
from AccessControl import ModuleSecurityInfo
from helpers import isRequestView
from Issue import issueFieldVisibility
from Permissions import *
from config import SKINS_DIR, GLOBALS, PROJECTNAME

def application_folder_container_filter(objectManager):
    """ container_filter attribute for ApplicationCategory instance
    
    This controls whether Add Application folder is shown in user's add menu. 
    By default, only Managers with permission MANAGE_USABILITY_ITEMS_PERMISSION can
    create new application folders 
    """
    #print "omanager:" + str(objectManager)
    #print "omanagerClass:" + str(objectManager.__class__)
    return True

def initialize(context):
    """ Register Usability product Archetypes and tune their permissions        
    
    Also register functions which can be called from TAL scripts.
    
    Parameters:
        
    @context Zope/App/ProductContext instance
    
    """
    
    registerDirectory(SKINS_DIR, GLOBALS)

    ##Import Types here to register them
    import Application
    import ApplicationSuperCategory
    import ApplicationSubCategory
    import Issue
    import UsabilityTool
    

    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)

    # Create factories for ApplicationCategorys
    utils.ContentInit(
        PROJECTNAME + ' Content',
        content_types      = content_types,
        permission         = VIEW_USABILITY_ITEMS_PERMISSION,
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)
        
    tools = (UsabilityTool.UsabilityTool,)
    utils.ToolInit(UsabilityTool.UsabilityTool.meta_type, tools=tools,
                       icon='tool.gif',
                       ).initialize(context)

    # Allow embedded TAL scripts to call functions in this module
    #allow_module("Products.Usability")
    ModuleSecurityInfo("Products.Usability").declarePublic("isRequestView")
    ModuleSecurityInfo("Products.Usability").declarePublic("issueFieldVisibility")
    
    # Fine tune creation permissions 
    # and now give it some extra permissions so that i
    # can control them on a per class limit
    
    for i in range(0,len(content_types)):
        perm='Add '+ capitalize(ftis[i]['id'])+'s'
        methname='add'+capitalize(ftis[i]['id'])
        meta_type = ftis[i]['meta_type']
            
        context.registerClass(
            meta_type=meta_type,
            constructors = (
                            getattr(locals()[meta_type],'add'+capitalize(meta_type)),
                               )
            , permission = perm
            )
    
    # Register creation permissions                                 
    
    context.registerClass(
            meta_type="Issue",
            constructors = (getattr(locals()["Issue"],'addIssue'),),
            permission = ADD_ISSUES_PERMISSION
            )    
    
    context.registerClass(
            meta_type="Application",
            constructors = (getattr(locals()["Application"],'addApplication'),),
            permission = ADD_ISSUES_PERMISSION
            )    
            
    context.registerClass(meta_type="ApplicationSubCategory",
            constructors = (getattr(locals()["ApplicationSubCategory"],'addApplicationSubCategory'),),
            permission = MANAGE_USABILITY_ITEMS_PERMISSION)            

    context.registerClass(meta_type="ApplicationSuperCategory",
            constructors = (getattr(locals()["ApplicationSuperCategory"],'addApplicationSuperCategory'),),
            permission = MANAGE_USABILITY_ITEMS_PERMISSION)                        
    
    # ApplicationCategory.addApplicationCategory can only be executed with
    # MANAGE_USABILITY_ITEMS_PERMISSION as stated in application_folder_container_filter 
    # and ApplicationCategory.invokeFactory later. EDIT_USABILITY_ITEMS_PERMISSION 
    # can create applications though
    #context.registerClass(meta_type="ApplicationCategory",
    #        constructors = (getattr(locals()["ApplicationCategory"],'addApplicationCategory'),),
    #        permission = EDIT_USABILITY_ITEMS_PERMISSION,
    #        container_filter = application_folder_container_filter)
    #context.registerClass(meta_type="ApplicationCategory",
    #        constructors = (getattr(locals()["ApplicationCategory"],'addApplicationCategory'),),
    #        permission = EDIT_USABILITY_ITEMS_PERMISSION)            


            



