"""

  Usability application category class
  Copyright 2005 Mikko Ohtamaa

  Short, unorganized functions, to perform crucial tasks which 
  are not big enough to deserve their own modules.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as described
  in licence.txt

"""

import types
from AccessControl.Permission import Permission
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import DisplayList


def getValues(self, prop_name):
    ptool = getToolByName(self, 'portal_properties', None)
    if ptool and hasattr(ptool, 'my_properties'):
        return ptool.my_properties.getProperty(prop_name, None)
    else:
        return None
    
def makeDisplayList(values=None):
    if values and type(values) not in [type([]), type(())]:
        values = (values,)
    if not values: values = []
    results = [['','Select']]
    for x in values:
        results.append([x,x])
    values_tuple = tuple(results)
    return DisplayList(values_tuple)

def getDisplayList(self, prop_name=None):
    return makeDisplayList(getValues(self, prop_name))
    
def addPermissionsForRole(context, role, wanted_permissions):
    """ Add (inheritable) permissions for a role for the context
    
    Parameters:
        @param context Portal object (portal itself, or Archetypes item, inherited from RoleManager)
        @param role role name, as a string
        @param wanted_permissions tuple of permissions (string names) to add for the role    
        
    All wanted_permissions lose their acquiring ability
    """        
            
    #print "Doing role:" + role + " perms:" + str(wanted_permissions)
    for p in context.ac_inherited_permissions(all=True):
        name, value = p[:2]
        p=Permission(name,value, context)        
        roles=list(p.getRoles())
            
        #print "Permission:" + name + " roles " + str(roles)
        if name in wanted_permissions:
            if role not in roles:
                roles.append(role)
            p.setRoles(tuple(roles))  
    
    
def stripPermissionForRole(context, role, permissions):
    """
    """
    # TODO: Code goes here
    pass
    
def doNotAcquirePermissions(context, permissions):
    """ Remove listed permissions from permission acquire 
        
    @param permissions tuple, these permissions aren't inherited anymore
    """
        
    # Get a list of permissions which are currently acquired
    now_acquired_permissions = []
    
    list_type = type([])
    tuple_type = type([])
    
    for p in context.ac_inherited_permissions(1):
        name, value = p[:2]
        p=Permission(name,value,context)
        roles=p.getRoles(default=[])
    
        # if roles are given as a list, then acquired flag is set
        # if they are given as a tuple, acquire flag is not set
        # kinky hacks they do, oh
        if type(roles) is tuple_type:
            now_acquired_permissions.append(name)
                        
    new_acquired_permissions = filter( lambda x: x not in permissions, now_acquired_permissions)
        
    context.manage_acquiredPermissions(new_acquired_permissions)
    
    
def forceWorkflowAction(workflow, ob, action, comment='', **kw):
    """ Do unpriviledged workflow state change
    
    Used to set initial object state to public.
    
    Code copied from DCWorkFlow.doActionFoR
    """
    kw['comment'] = comment
    sdef = workflow._getWorkflowStateOf(ob)
    if sdef is None:
        raise WorkflowException, 'Object is in an undefined state'
    if action not in sdef.transitions:
        raise Unauthorized(action)
    tdef = workflow.transitions.get(action, None)
    #if tdef is None or tdef.trigger_type != TRIGGER_USER_ACTION:
    #    raise WorkflowException, (
    #        'Transition %s is not triggered by a user action' % action)
    #if not self._checkTransitionGuard(tdef, ob, **kw):
    #    raise Unauthorized(action)
    workflow._changeStateOf(ob, tdef, kw)
    

def dumpPermissionsAndRoles(user, context):
    """ Print permission to role map to stdout
    
    List all permissions for the context object
    
    - Permission name
    
    - Is permissions acquired (inherit from the parent)
    
    - Locally set roles which have this permission    
    """
    
    list_type = type([])
    tuple_type = type([])
    
    all_permissions = context.ac_inherited_permissions(all=True)
    
    for p in all_permissions:
        name, value = p[:2]
        p=Permission(name,value,context)
        roles = p.getRoles()
        
        if type(roles) == list_type:
            print "Permission:" + name + " [Acquired] " + str(roles) 
        else:
            print "Permission:" + name + " [not acq] " + str(roles) 
            
def dumpRolesAndPermissions(user, context):
    """ Prints roles and permissions available for the user to the standard output 
    
    Parameters:
        
        @param context Archetypes item (or any item inherited from RoleManager)
        @param user GroupUserFolder/GRUFUser instance (getSecurityManager().getUser())
    
    """
    
    print "User class:" + str(user.__class__)
    print "Roles:" + str(user.getRoles())
    
    for role in user.getRoles():
        dumpRole(role, context)
        pass
    
def dumpRole(role, context):
    """ Prints role and it's permissions in context a to standard output 
    
        Uses same format as used on ZMI security page
        
        - If permission is printed it is acquired
        
        - If permissions is printed and it has selected set then it is set locally
    
        @param context Archetypes item (or any item inherited from RoleManager)
        @param role Role name as a string
    
    """
    print "Role name:" + role
    
    permissions = context.permissionsOfRole(role)
    for permission in permissions:
        print "Permission:" + str(permission)    


def isRequestView(object, wantedView):
    """ Checks if a widget should be shown in a certain view

    This function is used in Archetypes widget.condition TALES expressions to
    determine if a widget should be shown in a certain view.

    Parameters are
    - object: FSPageTemplate
    - wantedView: string or tuple (list of strings). This is given in actions definition
    - exclusive: boolean

    object.id should contain the name of a template which was used in this request.

    Return:
    - single string wantedView: true if object.id == wantedView
    - tuple wantedView: true if object.id is one of wantedView

    If object.id matches some entry wantedView then return true and thus show the widget in this view.


    Example:

            widget=ReferenceWidget(description="Duplicates",
                addable=1,
                destination="/",
                condition="python: modules['Products.Usability'].isRequestView(object, 'issue_add_view')"
            ),

            condition="python: modules['Products.Usability'].isRequestView(object, ('base_view', 'base_edit'))"

            or exclusive:

            condition="python: not modules['Products.Usability'].isRequestView(object, 'base_edit')"


    Note:
    - Available objects for condition TALES expressions are listed in CMFCore/Expression.py.
    - For each view, you need to copy Archetypes/skins/base_edit.cpt or base_view.pt to your skins directory
      as a view_name.pt
    - You must define custom views to your AT class actions variable:

          actions = ({'id': 'my_view',
                    'name': 'My View',
                    'action': 'string:${object_url}/this_is_the_string_compared_as_wanted_view',
                    'permissions': (VIEW_USABILITY_ITEMS_PERMISSION,)
                    },)

    - In product initialize() function you must give AccessControl public access to
      isWantedView function or Zope won't allow calling the function from a script.
      In __init__.py:

            from AccessControl import ModuleSecurityInfo
            from helpers import isRequestView

            def initialize(context):
                 ModuleSecurityInfo("Products.Usability").declarePublic("isRequestView")

    """
    if(type(wantedView) == tuple):
        return object.id in wantedView
    else:
        return(wantedView == object.id)


