"""
  Usability product unit tests
    
  Copyright 2005 Mikko Ohtamaa

  See class comments for more details

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as described
  in licence.txt

    
"""

import inspect
import os, sys

# Setup some environment variables here
# Allows unit test launching directly from IDE
os.environ['EVENT_LOG_FILE'] = os.path.join(os.getcwd(), 'zLOG.log')
os.environ['EVENT_LOG_SEVERITY'] = '-300'
os.environ['SOFTWARE_HOME'] = "..\..\..\..\Zope\lib\python"
os.environ['SOFTWARE_HOME'] = "..\..\..\..\Zope\lib\python"

print "log file:" + os.environ['EVENT_LOG_FILE']

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))    

import AccessControl
from AccessControl import getSecurityManager, ClassSecurityInfo, Unauthorized

# Set verbose security policy so we have more detailed information
# about security violations
from AccessControl.SecurityManager import setSecurityPolicy
from AccessControl.ImplPython import ZopeSecurityPolicy
setSecurityPolicy(ZopeSecurityPolicy(verbose=True))

from Testing import ZopeTestCase
from Testing.ZopeTestCase.ZopeLite import *
from Products.CMFPlone.tests import PloneTestCase
from Products.Usability.SampleContent import installContent
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.Usability.helpers import dumpRolesAndPermissions, dumpPermissionsAndRoles, dumpRole
from Products.CMFCore.Expression import Expression
from Products.PageTemplates.Expressions import getEngine
from Products.CMFPlone.tests.PloneTestCase import PloneTestCase

from Products.Usability.config import \
    PROJECTNAME, \
    GLOBALS, \
    USABILITY_MANAGER_ROLE, \
    USABILITY_EDITOR_ROLE
    
# Issue need ATVocabularyManager for the feature management
ZopeTestCase.installProduct("ATVocabularyManager")
ZopeTestCase.installProduct("Usability")


def executeUntrustedExpression(condition, mappings):
    """ A helper function to run code with restricted rights
    
    DO NOT USE THIS: Use PloneTestCase.setRoles for security testing
    
    Emulates evaluating TALES expressions 
    
    @param condition TALES expression to evaluate, e.g. "python: object.setTitle('blaa')"
    
    @param mappings Dictionary of available local variables for expressions, e.g.: 
       data = {
        'object':       object,
        'folder':       folder,
        'portal':       portal,
        'nothing':      None,
        'request':      getattr( object, 'REQUEST', None ),
        'modules':      SecureModuleImporter,
        'member':       member,
        }
    
    """        
    ec = getEngine().getContext(mappings)
    return Expression(condition)(ec)
    
class UsabilityTest(PloneTestCase):
    """ Unit testing class for Usability product
    
    Tested with Plone 2.1.0
    
    Tests cover
    - Adding and editing usability issues
    - Issue management security
    - Workflow management
    - Deleting items    
    
    """
        
    def createMember(self, id, pw, email, roles=('Member',)):
        pr = self.portal.portal_registration
        member = pr.addMember(id, pw, roles, properties={ 'username': id, 'email' : email })
        return member

    def createGroup(self, id):
        pg = self.portal.portal_groups
        pg.addGroup(id, '', (), ())
        return pg.getGroupById(id)
    
    def install(self):
        """ Run quick installers for products needed in this unit test """
        self.loginPortalOwner()
        qi = self.portal.portal_quickinstaller        
        qi.installProduct('ATVocabularyManager')
        qi.installProduct('Usability')


    def setupSecurityContext(self,):
        """ Setup test user and groups

        Create one eidtor user and 3 ordinal users
        
        """
        self.logout()
        self.loginPortalOwner()

        #create a bunch of users
        self.user1 = self.createMember('user1', 'abcd4', 'abc@domain.tld', roles=('Member',))
        self.user2 = self.createMember('user2', 'abcd4', 'abc@domain.tld', roles=('Member',))
        self.user3 = self.createMember('user3', 'abcd4', 'abc@domain.tld')
        self.editor = self.createMember('editor', 'abcd4', 'abc@domain.tld', roles=('Member', 'Manager'))
        
        self.createMemberarea('user1')
        self.createMemberarea('user2')
        self.createMemberarea('user3')
        self.createMemberarea('editor')
        
        #self.failUnless('user1' in self.portal.Members.objectIds())
        #self.failUnless('user2' in self.portal.Members.objectIds())
        #self.failUnless('user3' in self.portal.Members.objectIds())

        #just in case ...
        self.logout()
        self.login('user1')
        self.folder = self.portal.portal_membership.getHomeFolder('user1')
            
    def createApplication(self,appid):
        # Login
        user = self.login("editor")
        home = self.membership.getHomeFolder()

        appfolderid = "appfolder"

        # Create application folder
        self.membership.getHomeFolder().invokeFactory(
            type_name = 'ApplicationSubCategory',
            id = appfolderid
            )
        appfolder = getattr(home, appfolderid)

        # Content creation
        appfolder.invokeFactory(
            type_name = 'Application',
            id = appid
            )
        self.failUnless(appid in appfolder.objectIds(), "Object has not been created or not with the right id")
        return getattr(appfolder, appid)

    def createIssue(self, application, issueid):

        # Login
        self.login("editor")

        application.invokeFactory(
            type_name = 'Issue',
            id = issueid,
            title="Blaa",
            solution="Bloo"
            )
        self.failUnless(issueid in application.objectIds(), "Object has not been created or not with the right id")
        issue =  getattr(application, issueid)

        return issue


    def afterSetUp(self):
        PloneTestCase.afterSetUp(self)

        #some usefull properties
        self.catalog = self.portal.portal_catalog
        self.workflow = self.portal.portal_workflow
        self.membership = self.portal.portal_membership

        # quick install required products
        self.install()

        #setup users and groups
        self.setupSecurityContext()

        # Try to do "double install" which should not cause
        # error, but silent fail
        # installContent(self.portal, None)

    def test_00_install(self):
        """ See if we have been installed correctly """
        types = self.portal.portal_types.objectIds()
        for content in ('Application','Issue','ApplicationSubCategory'):
            self.failUnless(content in types, "Content type not propeply installed:" + content)

    def test_01_createApplication(self):
        """ Creates an applcation and checks if it initial rights and state are correct """
        
        self.login("editor")
        self.setRoles(("Manager",))        
        
        root = self.portal        
        root.invokeFactory(type_name='ApplicationSubCategory', id="appfolder")
        
        appfolder = root.appfolder
        appfolder.invokeFactory(type_name="Application", id="app")        
                    
        # check that application is public by default
        workflowTool = self.portal.portal_workflow        

        #print "state:" + str(workflowTool.getStatusOf("application_workflow", root.appfolder.app))
        
        self.failUnless(workflowTool.getStatusOf("application_category_workflow", root.appfolder)["review_state"] == "published",
            "ApplicationSubCategory was not public by default")            
        
        self.failUnless(workflowTool.getStatusOf("application_workflow", root.appfolder.app)["review_state"] == "published",
            "Application was not public by default")
            
        # test that manager is able to change data        
        appfolder.processForm(data=True)
        appfolder.app.processForm(data=True)
        
        self.logout()
        self.login("user1")
        self.setRoles(("Member",))
        
        # test that member is able to change data
        appfolder.app.processForm(data=True)
        

    def test_02_createIssue(self):
        app = self.createApplication("testapp");
        issue = self.createIssue(app, "testissue")    

        issue2 = self.createIssue(app, "testissue2")
        issue3 = self.createIssue(app, "testissue3")

        issue2 = getattr(app, "testissue2")        

    def test_04_security(self):
        """ Check that default permissions allow normal usage without security leaks
        
        TODO: Execute guarded code using self.setRoles instead of 
              through lambda functions and security manager instead calling
              TAL engine
        
        """                
                
        # Login as editor
        self.login("editor")
        self.setRoles(("Manager",))
        
        root = self.portal
                            
        appfolderid = "appfolder"

        # Create application folder
        root.invokeFactory(type_name='ApplicationSubCategory', id=appfolderid)        
        root.invokeFactory(type_name='ApplicationSuperCategory', id="superfolder")
        appfolder = root.appfolder
        superfolder = root.superfolder
        
        # Create two applications 
        appfolder.invokeFactory(type_name = 'Application', id="app")            
        appfolder.invokeFactory(type_name = 'Application', id="app2")                              
        appfolder.app.invokeFactory(type_name="Issue", id="issue")
        
        # now switch to normal member
        self.logout()                
        self.login("user1")
        
        self.setRoles(("Member",))
                
        #dumpRolesAndPermissions(getSecurityManager().getUser(), appfolder)
        #
        #dumpRole("Member", self.portal)
        
        # check if we are allowed to view folder contents
        appfolder.listFolderContents()
        appfolder.app.listFolderContents()               
                
        #dumpRolesAndPermissions(getSecurityManager().getUser(), self.app)
        
        # check if we are allowed to create issue under application created by editor
        appfolder.app.invokeFactory(type_name="Issue", id="issue2")
                
        #print "Dumping permissions for appfolder:"
        #dumpPermissionsAndRoles(getSecurityManager().getUser(), appfolder)
                
        # check if we are allowed to create an application
        appfolder.invokeFactory(type_name='Application',id = "app3")  
        appfolder.app3.invokeFactory(type_name="Issue", id="issue3")
        
        data = { "appfolder" : appfolder }
        #expr = "python: appfolder.invokeFactory(type_name='Application',id = 'app3')"
        #executeUntrustedExpression(expr, data)

        expr = "python: appfolder.app3.invokeFactory(type_name='Issue', id='issue4')"
        executeUntrustedExpression(expr, data)
        
        
        # editing app folders shouldn't be allowed
        try:
           #data = { "appfolder" : appfolder }
           #expr = "python: appfolder.setTitle('bloo')"
           #executeUntrustedExpression(expr, data)
           #self.fail("Member was able to edit ApplicationSubCategory")
           appfolder.setTitle('bloo')
        except Unauthorized, e: 
            pass   
        
                
        # creating appfolders shouldn't be allowed
        #try:
        #   root.invokeFactory(type_name='ApplicationSubCategory', id="appfolder2")
        #   self.fail("Normal user was able to create application folder")
        #except Exception, e: 
        #    if isinstance(e, AssertionError):
        #         raise e
        
        # check if another user can access items created by user1         
        self.logout()                
        self.login("user2")
        self.setRoles(("Member",))
        
        # Check we can edit Issues created by others
        actionTool = root.portal_actions
        actions = actionTool.listFilteredActionsFor(appfolder.app.issue)
        
        hasView = False
        hasEdit = False
        for objectAction in actions["object"]:
            if objectAction["name"] == "View": 
                hasView = True
            if objectAction["name"] == "Edit": 
                hasEdit = True
                
        self.failUnless(hasView, "View action missing for issue created by another user")
        self.failUnless(hasEdit, "Edit action missing for issue created by another user")
                
        appfolder.app3.listFolderContents()
        appfolder.app3.invokeFactory(type_name="Issue", id="issue5")
        appfolder.app3.issue3.setName( "Set by another user")
        actions2 = actionTool.listFilteredActionsFor(appfolder.app3.issue3)
        #print str(actions)
        
        # editing issues should be allowed
        data = { "issue" : appfolder.app.issue }
        executeUntrustedExpression("python: issue.setTitle('bloo')", data)
                                                      
        # now switch anonymous                
        self.logout()
        
        self.setRoles(("Anonymous",))
        
        appfolder.listFolderContents()
        appfolder.app.listFolderContents()
        appfolder.app3.listFolderContents()

        # creating appfolders shouldn't be allowed                
        try:
           superfolder.invokeFactory(type_name='ApplicationSubCategory', id="appfolder2")
           self.fail("Anonymous was able to create application folder")
        except Unauthorized, e: 
            pass

        # creating apps shouldn't be allowed
        try:
           appfolder.invokeFactory(type_name='Application', id="app4")
           self.fail("Anonymous was able to create application")
        except Unauthorized, e: 
            pass
            
        # editing apps shouldn't be allowed
        try:
           data = { "app" : appfolder.app }
           expr = "python: app.setTitle('bloo')"
           executeUntrustedExpression(expr, data)
           self.fail("Anonymous was able to edit Application")
        except Unauthorized, e: 
            pass      
        
        # creating issues shouldn't be allowed
        try:
           appfolder.app.invokeFactory(type_name='Issue', id="issue4")
           self.fail("Anonymous was able to create Issue")
        except Unauthorized, e: 
            pass      
            
        # editing issues shouldn't be allowed         
        data = { "issue" : appfolder.app.issue }       
        try:           
           executeUntrustedExpression("python: issue.setTitle('bloo')", data)           
           self.fail("Anonymous was able to edit Issue (via accessor)")
        except Unauthorized, e: 
            pass      
                        
        try:           
           executeUntrustedExpression("python: issue.setRootCause('bloo')", data)           
           self.fail("Anonymous was able to edit Issue (via accessor)")
        except Unauthorized, e: 
            pass      
            
                                        
    def test_05_workflow(self):
        """ Check that we can switch between workflow 
        
        Check that the state of sealed items cannot be changed
        
        TODO: Only states in_progress and sealed are tested
        
        """ 
        root = self.portal        
        workflowTool = root.portal_workflow
                
        # Login as editor
        self.login("editor")                
                        
        # Create application folder
        root.invokeFactory(type_name='ApplicationSubCategory', id="appfolder")
        appfolder = root.appfolder
                
        # now switch to normal member
        self.logout()                
        user = self.login("user1")
        
        # Create a application
        appfolder.invokeFactory(type_name = 'Application', id="app")
        
        # Create issue
        appfolder.app.invokeFactory(type_name="Issue", id="issue")            
                            
        flows = workflowTool.getWorkflowsFor(appfolder.app.issue)
        
        # Check issue gets correct workflow
        found = False        
        for flow in flows:
            if flow.id == "issue_workflow":
                found = True
        
        self.failUnless(found, "Default workflow for issues was " + workflowTool.getWorkflowsFor(appfolder.app.issue)[0].id)
        
                
        # sealing items by user shoudln't be allowed
        try:
            workflowTool.doActionFor(appfolder.app.issue, "seal")
            self.fail("Normal user was able to seal item")
        except: 
            pass
            
            
        #dumpRolesAndPermissions(getSecurityManager().getUser(), appfolder.app.issue)
        #dumpRole("Owner", appfolder.app.issue)
        
        self.logout()                
        self.login("editor")
        
        # Ensure we have Seal and Reopen actions listed in UI workflow menu
        actions = workflowTool.getActionsFor(appfolder.app.issue)

        # All state transitions should be available
        # (excluding inital state)
        self.failUnless(len(actions) == 4, "Had %d states" % (len(actions),) )
        
        # Check also that we have action available through action tool
        actionTool = root.portal_actions
        actions = actionTool.listFilteredActionsFor(appfolder.app.issue)                
        # TODO: Seal can be in any index
        #self.failUnless(actions["workflow"][0]["name"] == "Seal", "No Seal action listed for UI")
        
        # Check that Plone UI receives our state variable
        # TODO: review_state variable is hard coded to global_defines.pt in Plone 2.0.5
        # Other state varibles don't work. This might have been fixed in later versions.        
        info = workflowTool.getInfoFor(appfolder.app.issue, 'review_state', None);
        self.failUnless(info != None, "Plone 2.0.5 uses hard coded state variable review_state")
                                        
        # Seal test item 
        workflowTool.doActionFor(appfolder.app.issue, "seal")
        
        # Ensure we have Seal and Reopen actions listed in UI workflow menu
        actions = workflowTool.getActionsFor(appfolder.app.issue)
        # TODO: Reopen can be in any index
        #self.failUnless(actions[0]["name"] == "Reopen", "No Reopen action listed for UI")
        
        # Normal user shouldn't be allowed to change values when the item is sealed
        self.logout()                
        self.login("user1")
        
        #print "after sealing"        
        #dumpRolesAndPermissions(getSecurityManager().getUser(), appfolder.app.issue)
        #dumpRolesAndPermissions(getSecurityManager().getUser(), appfolder.app)
        #dumpRolesAndPermissions(getSecurityManager().getUser(), appfolder)
        #dumpRole("Anonymous", appfolder.app.issue)
        
        # user should not be able to see Edit action in view bar
        actions = actionTool.listFilteredActionsFor(appfolder.app.issue)
        hasView = False
        hasEdit = False
        for objectAction in actions["object"]:
            if objectAction["name"] == "View": 
                hasView = True
            if objectAction["name"] == "Edit": 
                hasEdit = True
        
        self.failUnless(hasView, "View action is not visible after sealing")
        self.failUnless(not hasEdit, "Edit is visible though the item is sealed")
        
        data = { "issue" : appfolder.app.issue }
                
        try:           
            executeUntrustedExpression("python: issue.setRootCause('bloo')", data)                               
            self.fail("Member user was able to edit sealed item")
        except Unauthorized,e:
            pass        
                            
        try:           
            executeUntrustedExpression("python: issue.setTitle('bloo')", data)
            self.fail("Member user was able to edit sealed item")
        except Unauthorized,e:
            pass                   

        self.logout()                
        self.login("editor")                
        workflowTool.doActionFor(appfolder.app.issue, "reopen")
        
        # Ensure that we propeply returned to initial in-progress state
        # TODO: Fix
        #actions = workflowTool.getActionsFor(appfolder.app.issue)
        #self.failUnless(len(actions) == 1)
        #self.failUnless(actions[0]["name"] == "Seal", "No Seal action listed for UI")
                
        self.logout()                
        self.login("user1")
        
        # now user should be able to edit item again
        appfolder.app.issue.setTitle("whooo")
        
        # Try seal reopened item 
        self.logout()                
        self.login("editor")                                        
        workflowTool.doActionFor(appfolder.app.issue, "seal")
        
    def test_06_delete(self):        
        """ Only managers should be able to delete an issues
        
        Only manager should be able to delete application/application folder
        """
        

        root = self.portal        
                
        # Login as editor
        self.login("editor")                
                        
        # Create application folder
        root.invokeFactory(type_name='ApplicationSubCategory', id="appfolder")
        appfolder = root.appfolder
                
        # now switch to normal member
        self.logout()                
        user = self.login("user1")
        # execute methods in member security context
        self.setRoles(("Member",))
        
        # Create a application
        appfolder.invokeFactory(type_name = 'Application', id="app")
        
        # Create issue
        appfolder.app.invokeFactory(type_name="Issue", id="issue")            
                
        self.logout()                
        self.login("user2")
        
        # execute methods in member security context
        self.setRoles(("Member",))
        
        # user2 tries to delete user1's item
        try:                       
            appfolder.app.manage_delObjects(ids=['issue'])
            self.fail("Deleted other users' issues")
        except Unauthorized,e:
            pass      
            
        
        self.logout()                
        self.login("user1")
        self.setRoles(("Member",))        
        
        # user1 should be able to remove his own issue
        del appfolder.app.issue
        
        # user1 should not be able to remove application he created
        try:           
            appfolder.manage_delObjects(ids=['app'])
            self.fail("Normal user deleted application")
        except Unauthorized,e:
            pass      
        
        self.logout()                
        self.login("editor")
        self.setRoles(("Manager",))              
        
        # editor should be able to remove anything
        del appfolder.app
        del self.portal.appfolder
            
    def test_07_voting(self):
        """ Test issue voting functionality
        
        Sealed items should not be votable 
        """

        root = self.portal        
        workflowTool = root.portal_workflow        
                
        # Login as editor
        self.login("editor")
        self.setRoles(("Manager",))
                        
        # Create application folder
        root.invokeFactory(type_name='ApplicationSubCategory', id="appfolder")
        appfolder = root.appfolder
                        
        # Create a application
        appfolder.invokeFactory(type_name = 'Application', id="app")
        
        # Create issue
        appfolder.app.invokeFactory(type_name="Issue", id="issue")            
        
        assert(getattr(appfolder.app.issue, "enableVoting") == True, "Voting was not enabled by default")
        
        # In-progress items should be voteable
        self.failUnless(appfolder.app.issue.isVoteable() == True, "In-progress items should be able to get votes")
        
        # Seal issue
        workflowTool.doActionFor(appfolder.app.issue, "seal")
        
        self.failUnless(appfolder.app.issue.isVoteable() == False, "Sealed items should not be able to get votes")
        

    def test_08_memory(self):
        from Products.Usability.MemoryDump import dump_memory
        dump_memory()
        
if __name__ == '__main__':
    framework()
